#!/usr/bin/perl

BEGIN
{
    ($dirprg, $main::SCRIPT) = $0 =~ /(.+)\/(.+)/;
    unshift(@INC, "$dirprg/../lib", "$dirprg/../../lib/");

}

use strict;
use ParamParser;
use Lucene;
use iANT::Cfg;
use iANT::Common;
require "iANT/Utils.pl";
use MIME::Base64;
use File::Temp;
use Cwd 'abs_path';




MAIN:
{

    my $o_param = New ParamParser(
                                   "GETOPTLONG",\&Usage,
                                   ("parser_cfg=s","store_dir=s" ,"outdir=s","output_field=s","light_entry_xml","outfile_suffix=s")
                                     );


    $o_param->AssertDefined('outdir');
    $o_param->AssertDirExists('store_dir');
    $o_param->AssertFileExists('parser_cfg');
    
    $o_param->SetBehaviour('use_substitution_table');
    $o_param = &iANT::Cfg::Load($o_param);
       
    my $outdir = $o_param->Get('outdir');
    mkdir $outdir if (! -d $outdir);

    
    
    my $parser = $o_param->Get('parser_cfg');
    my $store = $o_param->Get('store_dir');
    
    
    my $o_parser     = New ParamParser($parser);
    my $docid_field = $o_parser->Get('docid');
    $o_param->SetUnlessDefined('output_field', $o_parser->Get('default_out'));

    $o_parser->SetUnlessDefined('suffix',$docid_field);
    my $suffix = $o_parser->Get('suffix');
    $suffix =~ s/^\.//;
    
    my $output_field = $o_param->Get('output_field');
    
    
    my $lucene_searcher = $o_param->Get('lucene_searcher');
    my @a_docids = `$lucene_searcher --parser_cfg=$parser --store_dir=$store --list --query=$docid_field`;
    chomp @a_docids;

    my $fasta2entrydb_max_tree_subdirectories = $o_param->Get('fasta2entrydb_max_tree_subdirectories');
    
    my $subdir = 1;
    $subdir = sprintf ("%04d", $subdir);
    my $count = 0;


    foreach my $docid_value (@a_docids)
    {
            next if ($docid_value eq '');        
            if ($count == $fasta2entrydb_max_tree_subdirectories)
            {
                $subdir ++;
                $subdir = sprintf ("%04d", $subdir);
                $count = 0;
            }
            $count ++;
            

            mkdir "$outdir/$subdir" if (! -d "$outdir/$subdir");
            mkdir "$outdir/$subdir/$docid_value" if (! -d "$outdir/$subdir/$docid_value");
            
            my $cleaning_cmd = '';
            if ($o_param->IsDefined('light_entry_xml'))
            {
                $cleaning_cmd = '|perl -ne \'$_ =~ s/\n/__nl__/g;print\' | perl -ne \'$_ =~ s/<entry_features.+<\/entry_features>/<entry_features\/>/g;$_ =~ s/(\s*__nl__\s*)+/\n/g;print\' | xmllint --format -';
            }
            `$lucene_searcher  --parser_cfg=$parser --store_dir=$store --query=$docid_field:$docid_value --output_fields=$output_field --no_output_field_name $cleaning_cmd > $outdir/$subdir/$docid_value/$docid_value.$suffix`;
    }
}




sub Usage
{
    print STDOUT <<END;

Usage:   $0  --outdir=<existing output directory> --store=<lucene store> [--light]

    --light : delete entry_features element
    
END
}
