#!/usr/bin/perl

BEGIN
{
    ($dirprg, $main::SCRIPT) = $0 =~ /(.+)\/(.+)/;
    unshift(@INC, "$dirprg/../lib", "$dirprg/../../lib/");

}

use strict;
use ParamParser;
use Lucene;
use iANT::Cfg;
use iANT::Common;
require "iANT/Utils.pl";
use Cwd 'abs_path';
use Carp;

my $O_PARAM;

MAIN:
{

    my $create_index = 0;

    $O_PARAM = New ParamParser(
                               'GETOPTLONG',
                               (
                                "output_store=s","input_stores=s", "verbose"
                                 )
                                 );
    $O_PARAM->SetUsage(my $usage = sub {&Usage();});

    $O_PARAM->AssertDefined('input_stores');
    $O_PARAM->AssertDefined('output_store');


    $O_PARAM->SetBehaviour('use_substitution_table');
    $O_PARAM = &iANT::Cfg::Load($O_PARAM);

    my @a_input_stores = split (/[,\s+]/,$O_PARAM->Get('input_stores'));
    my @a_to_merge = ();
    foreach my $input_store (@a_input_stores)
    {
        $input_store = abs_path ($input_store);
        Carp::croak ("ERROR - input store  $input_store does not exist") if (! -d $input_store);
        if (-d $input_store)
        {
            opendir(ACCESSDIR, $input_store);
            my @a_store_files = readdir(ACCESSDIR);
            closedir(ACCESSDIR);

            #Even if the directory is empty, . and .. exists, so the array has a minimal length of 2
            if (!$create_index)
            {
                my $nb_cfs = grep (/\.cfs/, @a_store_files);
                if ($nb_cfs == 0)
                {
                   Carp::croak ("ERROR - No cfs file in this directory $input_store");
                }
            }
        }
        my $store = Lucene::Store::FSDirectory->getDirectory($input_store,0);
        print STDERR "Add index $input_store\n" if ($O_PARAM->IsDefined('verbose'));
        push (@a_to_merge, $store);
    }

    my $store_dir = abs_path($O_PARAM->Get('output_store'));
    

    Carp::croak ("ERROR - output store  $store_dir already exists") if (-d $store_dir);

    my $create_index = 1 if (!-d $store_dir);

    print STDERR "create_index $create_index $store_dir\n" if ($O_PARAM->IsDefined('verbose'));
    mkdir $store_dir if ($create_index);

    my $analyzer = new Lucene::Analysis::Standard::StandardAnalyzer();
    my $store = Lucene::Store::FSDirectory->getDirectory($store_dir, $create_index);
    my $writer = new Lucene::Index::IndexWriter($store, $analyzer, $create_index);
    $writer->setMergeFactor($#a_to_merge + 1);
    $writer->addIndexes(@a_to_merge); 
    $writer->optimize();
    $writer->close;
    



}

sub Usage
{

    print STDOUT <<END;
   
Usage:

   $0 --input_stores=</path/to/index1,/path/to/index2,/path/to/index3> --output_store=<directory> 

Description

   Merge a list of Lucene indices into one
   
 Mandatory
    --input_stores:   list of Lucene indices to merge  (coma separated)
    --output_store :   directory where merged Lucene index is stored.

                                

END
}
