#!/usr/bin/perl

BEGIN
{
    ($dirprg, $main::SCRIPT) = $0 =~ /(.+)\/(.+)/;
    unshift(@INC, "$dirprg/../lib", "$dirprg/../../lib/");

}

use strict;
use ParamParser;
use Lucene;
use iANT::Cfg;
require "iANT/Utils.pl";

use MIME::Base64;
use File::Temp;

use IO::File;

use sort 'stable';

use Proc::ProcessTable;
use Digest::MD5 qw(md5 md5_hex md5_base64);
use Cwd 'abs_path';
use Carp;

my $O_PARAM;
my @A_OUTPUT_FIELDS = ();
my @A_INDEX_FIELDS  = ();

MAIN:
{

    my $create_index = 0;

    $O_PARAM = New ParamParser(
                               'GETOPTLONG',
                               (
                                "parser_cfg=s",         "store_dir=s",
                                "query=s",              "output_fields=s",
                                "no_output_field_name", "outfile=s",
                                "limit=i",              "order_by=s",
                                "desc",                 "list_values",
                                "fuzzy",                "escape_special_characters",
                                "distinct_values",      "documents_separator=s",
                                "info", "verbose"
                                 )
                                 );
    $O_PARAM->SetUsage(my $usage = sub {&Usage();});
    $O_PARAM->AssertDefined('store_dir');
    $O_PARAM->SetUnlessDefined('documents_separator', "//\n");

    my $store_dir_string = $O_PARAM->Get('store_dir');
    if ($store_dir_string =~ /[\*\?]/)
    {
        $store_dir_string = `echo $store_dir_string`;
        chomp $store_dir_string;
    }
    my @a_store_dir = split(/[,\s]/, $store_dir_string);

    foreach my $store_dir (@a_store_dir)
    {
        if (!-d $store_dir)
        {
            print STDERR "ERROR:  $store_dir is not a directory\n";
            next;
        }
        &RemoveWeirdLockFile($store_dir);

        my $store;
        eval {$store = Lucene::Store::FSDirectory->getDirectory($store_dir, 0)};
        Carp::croak("ERROR [Main01]: $@") if ($@);
        &GetInfo($store) if ($O_PARAM->IsDefined("info"));
    }

    $O_PARAM->AssertFileExists('parser_cfg');
    my $parser_cfg = $O_PARAM->Get('parser_cfg');
    $O_PARAM->Update($parser_cfg, 'A');
    my ($ra_fields_to_parse, $rh_docid) = &GetLuceneFields4Xml($O_PARAM);
    &GetOutputFields($ra_fields_to_parse);
    &GetIndexFields($ra_fields_to_parse);

    $O_PARAM->SetUnlessDefined('output_fields', $O_PARAM->Get('default_out'));
    $O_PARAM->AssertDefined('query');

    my $analyzer_mod = $O_PARAM->Get('analyzer');
    my $analyzer     = new $analyzer_mod;

    $O_PARAM->SetBehaviour('use_substitution_table');
    $O_PARAM = &iANT::Cfg::Load($O_PARAM);

    my $unix_find = $O_PARAM->Get('unix_find');
    my $unix_gzip = $O_PARAM->Get('unix_gzip');
    my $unix_cat  = $O_PARAM->Get('unix_cat');

    my $query_string = $O_PARAM->Get('query');
    Carp::croak("ERROR - Fuzzy query is only available for single field search")
      if (($query_string !~ /^\s*(\S+):(\S+)\s*$/) && $O_PARAM->IsDefined('fuzzy'));

    $query_string = &EscapeLuceneSpecialCharaters($query_string, 'no')
      if ($O_PARAM->IsDefined('escape_special_characters'));

    #
    # Ici Test si query contient des tests numeriques ou des conditions &
    #	si oui, traitement special
    #	sinon, traitement classique

    my $output;
    if ($O_PARAM->IsDefined('outfile'))
    {
        my $out = $O_PARAM->Get('outfile');
        $output = new IO::File("+>$out") || Carp::croak("-- result_file -- cannot open the file : $out");
    }
    else
    {
        $output = *STDOUT;
    }

    #
    # TAG ROOT
    #
    my @a_output_fields = split(',', $O_PARAM->Get('output_fields'));
    my $xmltag;
    foreach my $output_field (@a_output_fields)
    {
        if (($O_PARAM->IsDefined($output_field . '_xmltag')) && ($O_PARAM->Get($output_field . '_xmltag') ne ''))
        {
            $xmltag = $O_PARAM->Get($output_field . '_xmltag');
            last;
        }
    }
    if (defined $xmltag and !$O_PARAM->IsDefined('list_values'))
    {
        print $output "<$xmltag>" . "\n";
    }

    #
    # FIN TAG ROOT
    #
    my $sort;    # = new Lucene::Search::Sort('RELEVANCE');

    if ($O_PARAM->IsDefined('order_by'))
    {
        my @a_sortfields = ();
        my @a_fields = split(',', $O_PARAM->Get('order_by'));
        foreach my $field (@a_fields)
        {
            $field =~ s/\s+//g;
            my $sortfield;

            if ($O_PARAM->IsDefined('desc'))
            {
                eval {$sortfield = new Lucene::Search::SortField($field, 1)};
                Carp::croak("ERROR [BuildSortField01]  $@") if ($@);
            }
            else
            {
                eval {$sortfield = new Lucene::Search::SortField($field)};
                Carp::croak("ERROR [BuildSortField02] $@") if ($@);
            }
            push(@a_sortfields, $sortfield);
            eval {$sort = new Lucene::Search::Sort(@a_sortfields)};
            Carp::croak("ERROR [BuildSortField03] $@") if ($@);
        }
    }

    my $limit = undef;
    $limit = $O_PARAM->Get('limit') if ($O_PARAM->IsDefined('limit'));

    foreach my $store_dir (@a_store_dir)
    {
        if (!-d $store_dir)
        {
            print STDERR "ERROR:  $store_dir is not a directory\n";
            next;
        }

        if ($O_PARAM->IsDefined('list_values'))
        {
            &ListValues($store_dir, $analyzer, $O_PARAM->Get('query'), $output, $O_PARAM);
            next;
        }
        my $store;
        eval {$store = Lucene::Store::FSDirectory->getDirectory($store_dir, 0)};
        Carp::croak("ERROR [Main02] $@") if ($@);

        my $searcher;
        eval {$searcher = new Lucene::Search::IndexSearcher($store)};
        Carp::croak("ERROR [Main03] $@") if ($@);

        my $parser;
        eval {$parser = new Lucene::QueryParser($O_PARAM->Get('default_search'), $analyzer)};
        Carp::croak("ERROR [Main04] $@") if ($@);

        my %h_seen  = ();
        my $rh_seen = \%h_seen;

        # Substitute spaces with '__' otherwise the tests will fail. There is probably a better way
        # to handle queries with "word1 word2 ..." but I have no other idea on the moment
        $query_string =~ s/ and / & /ig;
        $query_string =~ s/ or / \| /ig;
        $query_string =~ s/ not / ! /ig;
        my $tmp_query = $query_string;
        while ($tmp_query =~ /:"(.*?)"/g)
        {
            my $tmp_str = $1;
            my $mask    = $1;
            $tmp_str      =~ s/\s+/__/g;
            $query_string =~ s/\Q$mask\E/$tmp_str/;
        }

        my $local_query_string = $query_string;
        if ($local_query_string =~ /^(\s*\S+:\S+\s*[!&\|]*)+$/g)
        {
            my $query;
            if (($local_query_string =~ /^\s*(\S+):(\S+)\s*$/) && $O_PARAM->IsDefined('fuzzy'))
            {
                my $o_term = new Lucene::Index::Term($1, $2);
                my $o_fuzzy = new Lucene::Search::FuzzyQuery($o_term);
                $query = $o_fuzzy;
            }
            else
            {

                $local_query_string =~ s/\s+&\s+/ AND /g;
                $local_query_string =~ s/\s+\|\s+/ OR /g;

                #Lucene is weird with boolean operator. We put brackets around queries when there is an OR operator.
                #Lucene will understand A AND C OR B like X OR B. X = A AND C
                if ($local_query_string =~ /OR/)
                {
                    $local_query_string =~ s/ OR /\) OR \(/g;
                    $local_query_string = '(' . $local_query_string . ')';
                }

                $local_query_string =~ s/__/ /g;
                $query = $parser->parse($local_query_string);
            }
            my $hits = "";

            if (defined $sort)
            {
                eval {$hits = $searcher->search($query, $sort);};
            }
            else
            {
                eval {$hits = $searcher->search($query);};
            }
            if ($@ =~ /Too Many Clauses/)
            {
                print $output "TMC";
                exit(0);
            }
            my $num_hits = $hits->length();
            $num_hits = $limit if (defined $limit && ($limit < $num_hits));

            for (my $i = 0; $i < $num_hits; $i++)
            {
                my $document = $hits->doc($i);
                &PrintResult($document, $output);
            }
        }
        elsif ($local_query_string =~ /\s*\S+:\S+\s*/)
        {

            if ($local_query_string =~ /\|/)
            {
                my @a_queries = split('\|', $local_query_string);
                my $or = 1;
                foreach my $query (@a_queries)
                {
                    &doSearch($query, $store, $searcher, $parser, $output, $rh_seen, $or, $sort, $limit);
                }
            }
            else
            {
                &doSearch($local_query_string, $store, $searcher, $parser, $output, $rh_seen, 0, $sort, $limit);
            }

        }
        else
        {
            &GetDocuments($store, $local_query_string, $output, $rh_seen, 0, $sort, $limit);
        }
    }

    #
    # TAG ROOT
    #

    if (defined $xmltag and !$O_PARAM->IsDefined('list_values'))
    {
        print $output "</$xmltag>" . "\n";
    }
    if ($O_PARAM->IsDefined('outfile'))
    {
        $output->close if (defined $output);
    }

}

sub doSearch
{
    my ($query_string, $store, $searcher, $parser, $output, $rh_seen, $or, $sort, $limit) = @_;

    my $tmp_lucene_query = '';
    my ($lucene_query, $perl_query) = $query_string =~ /^(\s*\S+:\S+\s*)*(.*)$/;
    while ($perl_query =~ s/(\s*[&\|]\s+\S+:\S+\s*)//g)
    {
        $tmp_lucene_query .= $1;
    }
    if ($tmp_lucene_query ne '')
    {
        $lucene_query .= $tmp_lucene_query;
    }

    if ($lucene_query ne '')
    {
        $lucene_query =~ s/\s+&\s+/ AND /g;

        $lucene_query =~ s/__/ /g;
        my $query = $parser->parse($lucene_query);
        my $hits  = "";        
        if (defined $sort)
        {
                eval {$hits = $searcher->search($query, $sort);};
        }
        else
        {
            eval {$hits = $searcher->search($query);};
        }
        
        if ($@ =~ /Too Many Clauses/)
        {
            print STDERR "Exception catched!!";
            if ($0 =~ /iANT.idx.search2.pl/)
            {
                print STDERR "Your query has too many clauses, try to put some more letters!\n";
                return;
            }
            else
            {
                return "TMC";
            }
        }
        my $num_hits = $hits->length();
        $perl_query =~ s/^\s*\S\s+//;
        $perl_query =~ s/:/ =~ /g;
        for (my $i = 0; $i < $num_hits; $i++)
        {
            my $document    = $hits->doc($i);
            my $docid_value = $document->get($O_PARAM->Get('docid'));
            next if ($rh_seen->{$docid_value} and $or);
            if ($perl_query eq '')
            {
                &PrintResult($document, $output);
                $rh_seen->{$docid_value} = 1;
            }
            else
            {
                $perl_query =~ s/__/ /g;
                my $lucene_query_string = $perl_query;

                my @a_filters = split(/[&\|!]+/, $perl_query);
                foreach my $filter (@a_filters)
                {
                    if ($filter =~ /(\S+)\s*([><=~eqn!]+)\s*(\S+)\s*/)
                    {
                        my $field    = $1;
                        my $operator = $2;
                        my $value    = $3;
                        $field =~ s/\(|\)//g;

                        my $tested_value = $document->get($field);
                        $docid_value = $document->get($O_PARAM->Get('docid'));
                        next if ($rh_seen->{$docid_value} and $or);

                        if (   ($operator eq '=~')
                            || ($operator eq '!~')
                            || ($operator eq 'eq')
                            || ($operator eq 'ne'))
                        {
                            $tested_value =~ s/"/\\"/g;
                            $tested_value = '"' . $tested_value . '"';
                        }

                        my $new_filter = $filter;
                        $new_filter =~ s/$field/$tested_value/;

                        $lucene_query_string =~ s/$filter/$new_filter/;
                    }
                }

                $docid_value = $document->get($O_PARAM->Get('docid'));
                next if ($rh_seen->{$docid_value} and $or);
                $lucene_query_string =~ s/\s+\&{1}\s+/ \&\& /g;
                $lucene_query_string =~ s/\s+\|{1}\s+/ \|\| /g;

                if (eval($lucene_query_string))
                {
                    &PrintResult($document, $output);
                    $rh_seen->{$docid_value} = 1;
                }
            }
        }

    }
    else
    {
        &GetDocuments($store, $perl_query, $output, $rh_seen, $or, $sort, $limit);
    }
}

sub PrintResult
{
    my ($doc, $output) = @_;
    my $type;
    my $unix_gzip = $O_PARAM->Get('unix_gzip');

    my @a_output_fields = split(',', $O_PARAM->Get('output_fields'));
    my $xmltag;
    foreach my $output_field (@a_output_fields)
    {
        if (   ($O_PARAM->IsDefined($output_field . '_xmltag'))
            && ($O_PARAM->Get($output_field . '_xmltag') ne ''))
        {
            $xmltag = $O_PARAM->Get($output_field . '_xmltag');
            last;
        }
    }
    my @a_output_fields = split(',', $O_PARAM->Get('output_fields'));
    my $separator = $O_PARAM->Get('documents_separator');
    foreach my $output_field (@a_output_fields)
    {
        $output_field =~ s/\s//g;
        if (grep (/$output_field/, @A_OUTPUT_FIELDS))
        {
            my $output_field_value = $doc->get($output_field);
            if ($O_PARAM->Get("field:$output_field") =~ /unindexed\./i)
            {
                $output_field_value = &DecodeIfNecessary($output_field_value);
            }

            $separator = '' if ($O_PARAM->Get("field:$output_field") =~ /\.file$/i);
            $output_field_value =~ s/<\?xml.+\?>\n//
              if (($O_PARAM->Get("field:$output_field") =~ /\.file$/i) && (defined $xmltag));

            my $prefix = '';
            $prefix = "$output_field=" if (!$O_PARAM->IsDefined('no_output_field_name'));
            print $output "$prefix$output_field_value" . "\n";

        }
    }
    print $output $separator;

}

sub GetInfo
{
    my $store = shift;
    my $reader;
    eval {$reader = Lucene::Index::IndexReader->open($store)};
    Carp::croak("ERROR [GetInfo01] $@") if ($@);

    my $num_docs;
    eval {$num_docs = $reader->numDocs()};
    Carp::croak("ERROR [GetInfo02] $@") if ($@);

    print STDOUT "Index number of entries: $num_docs\n";

    exit;

}

sub GetDocuments
{
    my ($store, $query_string, $output, $rh_seen, $or, $sort, $limit) = @_;

    my $reader;
    eval {$reader = Lucene::Index::IndexReader->open($store)};
    Carp::croak("ERROR [GetDocuments01] $@") if ($@);

    #    my ($reader, $query_string, $output) = @_;
    my @a_documents = ();
    my $num_docs;

    eval {$num_docs = $reader->numDocs()};
    Carp::croak("ERROR [GetDocuments02] $@") if ($@);

    my $lucene_query_string = $query_string;

    #    print STDOUT $lucene_query_string."\n";
    for (my $i = 0; $i < $num_docs; $i++)
    {

        my $document;

        eval {$document = $reader->document($i)};
        Carp::croak("ERROR [GetDocuments03] $@") if ($@);

        my $docid_value;
        eval {$docid_value = $document->get($O_PARAM->Get('docid'))};
        Carp::croak("ERROR [GetDocuments04] $@") if ($@);

        next if ($rh_seen->{$docid_value} and $or);
        my $lucene_query_string = $query_string;

        my @a_filters = split(/[&\|]+/, $query_string);
        foreach my $filter (@a_filters)
        {
            if ($filter =~ /(\S+)\s*([><=~eqn!]+)\s*(\S+)\s*/)
            {
                my $field    = $1;
                my $operator = $2;
                my $value    = $3;
                my $tested_value;
                eval {$tested_value = $document->get($field)};
                Carp::croak("ERROR [GetDocuments05] $@") if ($@);
                if (($operator eq '=~') || ($operator eq '!~') || ($operator eq 'eq') || ($operator eq 'ne'))
                {
                    $tested_value =~ s/"/\\"/g;
                    $tested_value = '"' . $tested_value . '"';
                }

                my $new_filter = $filter;
                $new_filter          =~ s/$field/$tested_value/;
                $lucene_query_string =~ s/$filter/$new_filter/;
            }
        }

        $lucene_query_string =~ s/\s+\&{1}\s+/ \&\& /g;
        $lucene_query_string =~ s/\s+\|{1}\s+/ \|\| /g;
        if (eval($lucene_query_string))
        {

            &PrintResult($document, $output);
            $rh_seen->{$docid_value} = 1;
            my $printed_doc = scalar keys %{$rh_seen};
            last if (defined $limit && ($printed_doc == $limit));
        }
    }

}

sub GetOutputFields
{
    my $ra_fields_to_parse = shift;
    foreach my $rh_field_to_parse (@{$ra_fields_to_parse})
    {
        push(@A_OUTPUT_FIELDS, $rh_field_to_parse->{alias}) if ($rh_field_to_parse->{mode} !~ /unstored/i);
    }
    return;
}

sub GetIndexFields
{
    my $ra_fields_to_parse = shift;
    foreach my $rh_field_to_parse (@{$ra_fields_to_parse})
    {
        push(@A_INDEX_FIELDS, $rh_field_to_parse->{alias}) if ($rh_field_to_parse->{mode} !~ /unindexed/i);
    }
    return;
}

sub ListValues
{
    my ($store_dir, $analyzer, $field, $output, $o_param) = @_;
    if ($o_param->IsDefined('order_by') && ($o_param->Get('order_by') !~ /$field/))
    {
        Carp::croak("ERROR [ListValues00] order_by is only possible for field $field when using --list_values switch");
    }

    my $store;
    eval {$store = Lucene::Store::FSDirectory->getDirectory($store_dir, 0)};
    Carp::croak("ERROR [ListValues01] $@") if ($@);

    my $reader;
    eval {$reader = Lucene::Index::IndexReader->open($store)};
    Carp::croak("ERROR [ListValues02] $@") if ($@);

    my $num_docs;
    eval {$num_docs = $reader->numDocs()};
    Carp::croak("ERROR [ListValues03] $@") if ($@);

    my @a_results = ();
    my $result;
    for (my $i = 0; $i < $num_docs; $i++)
    {
        my $document;
        eval {$document = $reader->document($i)};
        Carp::croak("ERROR [ListValues04] $@") if ($@);

        eval {$result = $document->get($field)};
        Carp::croak("ERROR [ListValues05] $@") if ($@);

        if ($O_PARAM->Get("field:$field") =~ /unindexed\./i)
        {
            $result = &DecodeIfNecessary($result);
        }

        #		next if ($result eq '');
        push(@a_results, $result);
    }
    my @a_sorted_results = sort sort_alphanumeric @a_results;
    if ($o_param->IsDefined('desc'))
    {
        @a_sorted_results = reverse @a_sorted_results;
    }

    my $limit;
    $limit = $o_param->Get('limit') if ($o_param->IsDefined('limit'));

    if ($o_param->IsDefined('distinct_values'))
    {
        my %h_distincts = ();

        foreach my $cell (@a_sorted_results)
        {
            if (!defined $h_distincts{$cell})
            {
                print $output $cell . "\n";
                $h_distincts{$cell} = 1;

                last if (defined $limit && (scalar keys %h_distincts == $limit));

            }
        }
    }
    else
    {
        $limit = scalar @a_sorted_results unless (defined $limit);

        for (my $i = 0; $i < $limit; $i++)
        {
            print $output $a_sorted_results[$i] . "\n";
        }
    }
}

sub DecodeIfNecessary
{
    my $data = shift;

    return $data if ($data eq '');

    my $unix_base64 = $O_PARAM->Get('unix_base64');
    my $unix_cat    = $O_PARAM->Get('unix_cat');

    my $tmp_dir = $O_PARAM->Get('tmp_dir');
    my ($tmp_fh, $tmp_filename) = File::Temp::tempfile(DIR => $tmp_dir);
    print $tmp_fh $data;
    $tmp_fh->close;

    my $cmd     = "$unix_base64 -d $tmp_filename > $tmp_filename.decode";
    my $not_b64 = `($cmd) 2>&1`;

    chomp $not_b64;

    if ($not_b64 eq '')
    {

        my $b64_decoded = `$unix_cat $tmp_filename.decode`;
        chomp $b64_decoded;
        my $unix_gzip = $O_PARAM->Get('unix_gzip');
        $data = `$unix_gzip -cd $tmp_filename.decode`;
        chomp $data;
    }

    unlink $tmp_filename;
    unlink "$tmp_filename.decode";

    return $data;
}

sub sort_alphanumeric
{
    return $a <=> $b if ($a =~ /^\d+$/ and $b =~ /^\d+$/);
    return -1 if ($a eq '' or $a =~ /^\d+$/);
    return 1  if ($b eq '' or $b =~ /^\d+$/);
    return $a cmp $b if ($a !~ /\d/i and $b !~ /\d/i);

    # We split the words we want to compare and we compare each part
    my @a_splitted_a = split(/(\d+)/, $a);
    my @a_splitted_b = split(/(\d+)/, $b);
    my $index        = 0;
    my $test         = 0;
    foreach my $cell (@a_splitted_a)
    {
        if ($cell =~ /\d/ and $a_splitted_b[$index] =~ /\d/)
        {
            $test = $cell <=> $a_splitted_b[$index];
        }
        else
        {
            $test = $cell cmp $a_splitted_b[$index];
        }
        return $test if ($test != 0);
        $index++;
    }
    return 0;
}

sub RemoveWeirdLockFile
{

    my $store_dir = shift;

    my $tries = 5;
    $store_dir =~ s/\/$//;
    my $store_dir_md5 = md5_hex(abs_path($store_dir));
    my $try           = 0;
    while (
         ((-e '/tmp/lucene-' . $store_dir_md5 . '-commit.lock') || (-e '/tmp/lucene-' . $store_dir_md5 . '-write.lock'))
         && ($try <= $tries))
    {
        my $o_process_table = new Proc::ProcessTable;
        foreach my $o_process (@{$o_process_table->table})
        {
            if ($o_process->cmndline =~ /$store_dir/ && $o_process->pid != $$)
            {
                $try++;
                sleep 5;
            }
            else
            {
                unlink('/tmp/lucene-' . $store_dir_md5 . '-commit.lock') if (-e '/tmp/lucene-' . $store_dir_md5 . '-commit.lock');
                unlink('/tmp/lucene-' . $store_dir_md5 . '-write.lock') if (-e '/tmp/lucene-' . $store_dir_md5 . '-write.lock');
                print STDERR "[WARNING] a persistant lock file has been removed\n" if ($O_PARAM->IsDefined('verbose'));
                last;
            }
        }
    }

    if (   (-e '/tmp/lucene-' . $store_dir_md5 . '-commit.lock')
        || (-e '/tmp/lucene-' . $store_dir_md5 . '-write.lock'))
    {
        Carp::croak("a persitant lock file has been found and could not be removed:\n\t");
        my $o_process_table = new Proc::ProcessTable;
        foreach my $o_process (@{$o_process_table->table})
        {
            if ($o_process->cmndline =~ /$store_dir/ && $o_process->pid != $$)
            {
                print STDERR "[WARNING] a program is accessing to your index: " . $o_process->cmndline;
            }
        }
    }
}

sub Usage
{
    my $output_fields = join("\n\t\t\t\t", @A_OUTPUT_FIELDS);
    my $index_fields  = join("\n\t\t\t\t", @A_INDEX_FIELDS);
    my $default_ouput = $O_PARAM->Get('default_out');
    my $default_search = $O_PARAM->Get('default_search');

    print STDOUT <<END;
   
Usage:

   $0 --parser_cfg=<file> --store_dir=<directory> --query=<string> --output_fields=s
   
 Mandatory
    --parser_cfg:   parser configuration file (contains Analyzer name, fields to parse, ...)
    --store_dir :   directory where Lucene indeces are stored
    --query     :   Lucene query string (default search field: $default_search)
                    Search fields:
                                $index_fields
                                
 Optional
 
    --output_fields : comma separated fields to extract
                    default:    $default_ouput
                    values :    $output_fields
    --no_output_field_name : to avoid output_fields display
    --escape_special_characters : to escape special characters in Lucene query
			           list: \ + - ! ( ) { } [ ] ^ " ~ * ?
    --outfile : the output file
    
    --limit : limit number of retrieved documents
    
    --order_by : sort output according to a list of fields (comma separated)
    --desc : reverse sort
    
    --info	: index information
    --list_values : list all the values contained in the field given to --query
    			--query=accession --list_values : return all the accession ids
    --distinct_values : used with --list_values, this will delete double values
    --documents_separator : string separator used default: '//\\n'
    --fuzzy : fuzzy query ONLY AVAILABLE FOR SINGLE FIELD QUERY (field:fuzzy_value)
    
    --verbose

END
}
